import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn import preprocessing

le = preprocessing.LabelEncoder()

sns.set_style("whitegrid")
plt.style.use("fivethirtyeight")

dataDf = pd.read_csv("weight_data_with_headers.csv")
le.fit(dataDf['breed'])
print(le.classes_)
dataDf['breed'] = le.transform(dataDf['breed'])

print(dataDf.head())

sns.pairplot(dataDf)
plt.show()
sns.heatmap(dataDf.corr(), annot=True)
plt.show()

corr = dataDf.corr(method='pearson')
print(corr)

from sklearn.model_selection import train_test_split

X = dataDf[['breed', 'height', 'weight']]
y = dataDf['outcome']
X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.3, random_state=0)

from sklearn.linear_model import LogisticRegression

# LOGISTIC REGRESSION
clf = LogisticRegression(random_state=0, max_iter=2000, solver='lbfgs').fit(X_train, y_train)
predictions = clf.predict(X_test)
score = clf.score(X_test, y_test)
print("Logistic regression score: ", score)

from sklearn import metrics
cm = metrics.confusion_matrix(y_test, predictions)
print(cm)

plt.figure(figsize=(9,9))
sns.heatmap(cm, annot=True, fmt=".3f", linewidths=.5, square = True, cmap = 'Blues_r');
plt.ylabel('Actual label');
plt.xlabel('Predicted label');
all_sample_title = 'Accuracy Score: {0}'.format(score)
plt.title(all_sample_title, size = 15)
plt.show()

# SVM
from sklearn.pipeline import make_pipeline
from sklearn.preprocessing import StandardScaler

from sklearn.svm import SVC
clf = make_pipeline(StandardScaler(), SVC(gamma='auto'))
clf.fit(X, y)
score = clf.score(X, y, sample_weight=None)
print("SVM score: ", score)
predictions = clf.predict(X_test)
cm = metrics.confusion_matrix(y_test, predictions)
print(cm)

plt.figure(figsize=(9,9))
sns.heatmap(cm, annot=True, fmt=".3f", linewidths=.5, square = True, cmap = 'Blues_r');
plt.ylabel('Actual label');
plt.xlabel('Predicted label');
all_sample_title = 'Accuracy Score: {0}'.format(score)
plt.title(all_sample_title, size = 15)
plt.show()
